"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cookie = require("cookie");
const https = require("https");
class BruntHttp {
    /**
     *
     * @param debug set to true to log http response and errors to the console
     */
    constructor(debug) {
        this.debug = debug;
    }
    POST(data) {
        data.method = "POST";
        return this._http(data);
    }
    PUT(data) {
        data.method = "PUT";
        return this._http(data);
    }
    GET(data) {
        data.method = "GET";
        return this._http(data);
    }
    DELETE(data) {
        data.method = "DELETE";
        return this._http(data);
    }
    OPTIONS(data) {
        data.method = "OPTIONS";
        return this._http(data);
    }
    _http(data) {
        return new Promise((resolve, reject) => {
            //console.log("Timeout setting: ",waitTime/1000,"s");
            let options;
            options = {
                hostname: data.host,
                port: data.port,
                path: data.path,
                method: data.method,
                headers: {},
            };
            options.headers = {};
            //console.log(options.path);
            if (data.data) {
                data.data = JSON.stringify(data.data);
                options.headers["Content-Length"] = Buffer.byteLength(data.data);
                options.headers["Content-Type"] = "application/x-www-form-urlencoded; charset=UTF-8";
                options.headers.Origin = "https://sky.brunt.co";
                options.headers["Accept-Language"] = "en-gb";
                options.headers.Accept = "application/vnd.brunt.v1+json";
                options.headers["User-Agent"] = "Mozilla/5.0 (iPhone; CPU iPhone OS 11_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E216";
            }
            if (data.sessionId) {
                options.headers.Cookie = "skySSEIONID=" + data.sessionId;
            }
            if (this.debug) {
                console.log("[Brunt API Debug] HTTP Request:\n", options);
            }
            let resp = "";
            const req = https.request(options, (response) => {
                response.on("data", (chunk) => {
                    resp += chunk;
                });
                response.on("end", () => {
                    if (this.debug) {
                        console.log("[Brunt API Debug] Response in http:\n", resp);
                    }
                    try {
                        if (response.statusCode === 200 && resp.length > 1) {
                            resp = JSON.parse(resp);
                            const setcookie = response.headers["set-cookie"];
                            if (setcookie !== undefined) {
                                const cookies = cookie.parse(setcookie[0]);
                                resp.sessionId = cookies.skySSEIONID;
                                if (this.debug) {
                                    console.log("[Brunt API Debug] Session ID: ", cookies.skySSEIONID);
                                }
                            }
                        }
                        else if (response.statusCode === 200) {
                            // Empty 200 response
                            resp = true;
                        }
                        resolve(resp);
                    }
                    catch (e) {
                        if (this.debug) {
                            console.log("[Brunt API Debug] e.stack:\n", e.stack);
                            console.log("[Brunt API Debug] Raw message:\n", resp);
                            console.log("[Brunt API Debug] Status Code:\n", response.statusCode);
                        }
                        reject(e);
                    }
                });
            });
            req.on("error", (e) => {
                if (this.debug) {
                    console.log(`[${new Date()} Brunt API Debug] Error at req: ${e.code.trim()} - ${data.path}\n`);
                }
                reject(e);
            });
            // For POST (submit) state
            if (data.data) {
                req.write(data.data);
                //console.log(data.data);
            }
            req.end();
        });
    }
}
class BruntAPI {
    constructor(debug) {
        this.http = new BruntHttp(debug);
    }
    login(username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = {
                data: {
                    ID: username,
                    PASS: password,
                },
                path: "/session",
                host: "sky.brunt.co",
                port: "443",
            };
            const resp = yield this.http.POST(data);
            if (resp && resp.status && resp.status === "activate") {
                return resp;
            }
            throw new Error("Login failed");
        });
    }
    getThings(sessionId) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = {
                path: "/thing",
                host: "sky.brunt.co",
                port: 443,
                sessionId,
            };
            const resp = yield this.http.GET(data);
            return resp;
        });
    }
    getState(sessionId, thingUri) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = {
                path: "/thing" + thingUri,
                host: "thing.brunt.co",
                port: 8080,
                sessionId,
            };
            const resp = yield this.http.GET(data);
            return resp;
        });
    }
    changePosition(sessionId, thingUri, position) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = {
                data: {
                    requestPosition: position.toString(),
                },
                path: "/thing" + thingUri,
                host: "thing.brunt.co",
                port: 8080,
                sessionId,
            };
            const resp = yield this.http.PUT(data);
            return resp;
        });
    }
}
exports.BruntAPI = BruntAPI;
//# sourceMappingURL=bruntapi.js.map